/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.pdfconv;

import de.hendriklipka.pdfconv.config.ImageConfig;
import org.jpedal.PdfDecoder;

import java.awt.color.ColorSpace;
import java.awt.geom.AffineTransform;
import java.awt.image.*;

public class ImageProcessor
{
    private ImageConfig _config;

    private static final boolean COLOR_CONVERT=false;
    private boolean _verbose=false;
    private boolean _debug=false;

    public ImageProcessor(ImageConfig config)
    {
        _config=config;
    }

    public BufferedImage processImage(BufferedImage image)
    {
        BufferedImage targetImage=null;

        if (_debug)
            System.out.println("image: width="+image.getWidth()+", heigth="+image.getHeight());
        double heigth=image.getHeight();
        double width=image.getWidth();
        boolean flipImage=false;
        AffineTransform at=new AffineTransform();

        String version=PdfDecoder.version;
        double d=Double.parseDouble(version);
        if (d>1.81) // flipping the image is needed since jpedal version 1.81
        {
            if (_debug)
                System.out.println("flipping image");
            flipImage=true;
            at.scale(1, -1); // flip image upside-down
            at.translate(0, -image.getHeight());
        }
        boolean scaleImage=false;
        double scaleFactor=_config.scaleFactor;
        if (-1!=_config.maxHeigth||-1!=_config.maxWidth||1!=_config.scaleFactor)
        {
            if (_debug)
                System.out.println("checking for scale");
            if (-1!=_config.maxWidth&&_config.maxWidth<width)
            {
                d=_config.maxWidth/width;
                if (d<scaleFactor)
                    scaleFactor=d;
                scaleImage=true;
                if (_debug)
                    System.out.println("scale width="+scaleFactor);
            }
            if (-1!=_config.maxHeigth&&_config.maxHeigth<heigth)
            {
                d=_config.maxHeigth/heigth;
                if (_debug)
                    System.out.println("scale heigth="+d);
                if (d<scaleFactor)
                    scaleFactor=d;
                scaleImage=true;
            }
            if (1!=_config.scaleFactor)
                scaleImage=true;
            if (_debug)
                System.out.println("scale factor="+scaleFactor);
            at.scale(scaleFactor, scaleFactor);
        }
        if (flipImage||scaleImage)
        {
            if (_debug)
                System.out.println("doing transformation");
            AffineTransformOp ato=new AffineTransformOp(at, null);
            targetImage=ato.filter(image, null);
        }
        else
        {
            targetImage=image;
        }

        if (COLOR_CONVERT)
        {
            if (_debug)
                System.out.println("color conversion");
            if (_debug)
                System.out.println(targetImage.getColorModel().toString());
            ColorSpace cs=ColorSpace.getInstance(ColorSpace.CS_sRGB);
            ColorConvertOp cco=new ColorConvertOp(cs, null);
            targetImage=cco.filter(targetImage, null);
            if (_debug)
            {
                System.out.println("image: width="+targetImage.getWidth()+", heigth="+targetImage.getHeight());
                System.out.println(targetImage.getColorModel().toString());
            }
            ColorModel cm=targetImage.getColorModel();
            if (_debug)
                if (cm instanceof IndexColorModel)
                    if (_debug)
                        System.out.println("index color model");
        }
        return targetImage;
    }

    public void setVerbose(boolean verbose)
    {
        _verbose=verbose;
    }

    public void setDebug(boolean debug)
    {
        _debug=debug;
    }
}
